# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from uds.models import Provider
from django.core.management.base import CommandError
from django.db import transaction
from uds.core import services

from termidesk.management.commands.common.create_subcommand import CreateSubcommand


class CreateCommand(CreateSubcommand):
    name_of_service = 'поставщика ресурсов'

    BASE_ARGS = (
        'name', 'comments',
    )

    _factory = None

    types = [p.typeType for p in
             list(services.factory().providers().values())]

    def add_subparser(self, subparsers):
        create_parser = subparsers.add_parser(
            CreateSubcommand.SUBCOMMAND,
            help="Create service"
        )

        self.add_arguments_subcommand(create_parser)

        create_parser.add_argument(
            '--name', type=str, required=True,
            help="Имя объекта"
        )

        create_parser.add_argument(
            '--comments', type=str,
            help="Комментарий для объекта",
            default=''
        )

    def validate_base_data(self, data):
        """ Проверка существования провайдера по имени. """
        try:
            name = data['name']
            Provider.objects.get(name=name)
            error = ('Провайдер с таким именем {} '
                     'уже существует').format(name)
            raise CommandError(error)
        except Provider.DoesNotExist:
            pass

    @transaction.atomic()
    def create_service(self, provider_type, main_args, params):
        provider = Provider.objects.create(**main_args)
        provider.data_type = provider_type
        provider.data = provider.getInstance(params).serialize()
        provider.save()

    def get_service_factory(self):
        if self._factory is None:
            self._factory = services.factory()
        return self._factory