# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.core.management.base import (BaseCommand,
                                         CommandError,
                                         CommandParser)
from uds.models import Provider
from termidesk.management.commands.subcommands.prov.tpl.create import (
    CreateCommand
)


class IdCommand(BaseCommand):
    SUBCOMMAND = 'id'
    SUBPARSERS_DEST = 'id_subcommands'
    provider = None

    def get_item_by_uid(self, uid):
        try:
            self.provider = Provider.objects.get(uuid=uid)
        except Provider.DoesNotExist:
            raise CommandError('Not found provider by uid={}'.format(uid))

    def add_subparser(self, cmd, parent_subparsers):
        """
        Добавление вложенного парсера.
        :param cmd: Объект, которому необходимо добавить вложенные подкаманды
        :param parent_subparsers:
        :return:
        """
        self.id_parser = parent_subparsers.add_parser(
            self.SUBCOMMAND,
            help="Show items of a service",
        )

        self.id_parser.add_argument(
            '{id}', type=self.get_item_by_uid,
            nargs=1, help='Передайте уникальный код поставщика ресурсов')

        class SubParser(CommandParser):
            def __init__(self, **kwargs):
                super(SubParser, self).__init__(cmd, **kwargs)

        subparsers = self.id_parser.add_subparsers(
            title="subcommands",
            dest=self.SUBPARSERS_DEST,
            parser_class=SubParser
        )

        self.tpl_command = TplCommand()
        self.tpl_command.add_subparser(cmd, subparsers)

    def handle_subcommand(self, options, stdout, stderr):
        self.tpl_command.handle_subcommand(options, stdout, stderr,
                                           self.provider)


class TplCommand(BaseCommand):
    SUBCOMMAND = 'tpl'
    SUBPARSERS_DEST = 'tpl_subcommands'

    def add_subparser(self, cmd, parent_subparsers):
        tpl_parser = parent_subparsers.add_parser(
            self.SUBCOMMAND,
            help="Create service"
        )

        class SubParser(CommandParser):
            def __init__(self, **kwargs):
                super(SubParser, self).__init__(cmd, **kwargs)

        subparsers = tpl_parser.add_subparsers(
            title="subcommands",
            dest=self.SUBPARSERS_DEST,
            parser_class=SubParser
        )

        self.create_command = CreateCommand(self.style)
        self.create_command.add_subparser(subparsers)

    def handle_subcommand(self, options, stdout, stderr, provider):
        self.create_command.handle_subcommand(options, stdout,
                                              stderr, provider)
