# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.core.management.base import CommandError
from django.db import transaction
from uds.models import ServicesPoolGroup
from termidesk.management.commands.common.common_command import CommonCommand
from termidesk.management.commands.common.subcommand import Subcommand
from termidesk.management.commands.common.argtypes import \
    positive_int_type


class Command(CommonCommand):
    help = '''Создание группы рабочих мест.'''

    create_command = None

    def add_arguments(self, parser):
        super(Command, self).add_arguments(parser)
        self.create_command = CreateCommand(self.style)
        self.create_command.add_subparser(self.subparsers)

    def handle(self, *args, **options):
        subcommand = options.get('subcommand')

        if subcommand == self.create_command.SUBCOMMAND:
            return self.create_command.handle_subcommand(
                options,
                self.stdout,
                self.stderr
            )

        self.stdout.write(
            self.style.ERROR(
                'Не удалось обработать команду '.format(subcommand)
            )
        )


class CreateCommand(Subcommand):
    name_of_service = 'группы рабочих мест'

    SUBCOMMAND = 'create'

    BASE_ARGS = (
        'name', 'comments', 'priority'
    )

    _factory = None

    def handle_subcommand(self, options, stdout, stderr):

        self.validate_base_data(options)

        loud_work = not options['quiet']

        try:
            main_args = {arg: options[arg] for arg in self.BASE_ARGS
                         if arg in options}

            self.create_service(main_args)
            if loud_work:
                stdout.write(
                    self.style.SUCCESS('Создание {} прошло успешно'.format(
                        self.name_of_service
                    ))
                )
        except Exception as error:
            stdout.write(
                self.style.ERROR('Не удалось создать поставщик ресурсов. '
                                 + str(error))
            )

    def add_subparser(self, subparsers):
        create_parser = subparsers.add_parser(
            self.SUBCOMMAND,
            help="Create service"
        )

        self.add_arguments_subcommand(create_parser)

        create_parser.add_argument(
            '--name', type=str, required=True,
            help="Имя объекта"
        )

        create_parser.add_argument(
            '--comments', type=str,
            help="Комментарий", default=''
        )

        create_parser.add_argument(
            '--priority', type=positive_int_type,
            help="Приоритет", default=1,

        )

    def add_arguments_subcommand(self, parser):
        parser.add_argument(
            '--quiet', action='store_true', dest='quiet', default=False,
            help='Выводить только сообщения об ошибках',
        )

    @transaction.atomic()
    def create_service(self, main_args):
        return ServicesPoolGroup.objects.create(**main_args)

    def validate_base_data(self, data):
        """ Проверка существования сети по имени. """
        try:
            ServicesPoolGroup.objects.get(name=data['name'])
            error = ('Группа с таким именем {} '
                     'уже существует').format(data['name'])
            raise CommandError(error)
        except ServicesPoolGroup.DoesNotExist:
            pass
