# -*- coding: utf-8 -*-
'''
Created on 16 янв. 2019 г.

@author: dvd
'''
from __future__ import unicode_literals

# 2to3: Optimized imports: deleted json, websocket, socket, urllib

import websockify
import logging
from termidesk.core.util import wsptoken
import argparse


class TermideskProxyRequestHandler(websockify.ProxyRequestHandler):
    def __init__(self, retsock, address, proxy, *args, **kwargs):
        self._proxy = proxy
        websockify.ProxyRequestHandler.__init__(self, retsock, address, proxy,
                                                *args, **kwargs)

    def get_target(self, target_plugin):
        """
        Parses the path, extracts a token, and looks for a valid
        target for that token in the configuration file(s). Returns
        target_host and target_port if successful and sets an ssl_target
        flag.
        """
        connection_data = self._proxy._tokenDecoder.decode(self.path[2:])
        self.logger.debug("Decoded token data: %s", connection_data)
        target_host = connection_data['host']
        target_port = connection_data['port']
        self.server.ssl_target = connection_data['ssl_target']
        return (target_host, target_port)

class TermideskWebSocketProxy(websockify.WebSocketProxy):
    """"
    Обертка вокруг websockify прокси для работы с Termidesk
    """

    def __init__(self, tokenDecoder, logger = None, *args, **kwargs):
        self._tokenDecoder = tokenDecoder
        if logger:
            self._logger = logger
        else:
            self._logger = logging.getLogger('TermideskWebSocketProxy')
            
        super(TermideskWebSocketProxy, self).__init__(*args, **kwargs)

    def get_logger(self):
        return self._logger

if __name__ == '__main__':
    parser = argparse.ArgumentParser(description='Termidesk websocket proxy')
    parser.add_argument('--log', default='', help='Log file')
    parser.add_argument('--verbose', dest='verbose', action='store_true', help='Verbose output')
    parser.add_argument('--host', default='127.0.0.1', help='Listen ip address')
    parser.add_argument('--port', default=5099, type=int, help='Listen port')
    parser.add_argument('--verification-key', required=True, help='Path to token data verification key')
    
    args = parser.parse_args()
     
    with open(args.verification_key) as f:
        public_key = f.read()
    
    log_level = logging.INFO
    log_format = '[%(asctime)s] [%(process)s] [%(threadName)s] [%(levelname)s] [%(name)s] %(message)s'
    log_datefmt = '%Y-%m-%d %H:%M:%S %z'
    if args.verbose:
        log_level = logging.DEBUG
    if args.log:    
        logging.basicConfig(
            format = log_format,
            datefmt = log_datefmt,
            filename=args.log,
            level=log_level)
    else:
        logging.basicConfig(
            format = log_format,
            datefmt = log_datefmt,
            level=log_level)
    
    kwargs = {'token_plugin': 'TokenFile'}
    
    TermideskWebSocketProxy(
        listen_host=args.host,
        listen_port=args.port,
        source_is_ipv6=False,
        verbose=args.verbose,
        tokenDecoder=wsptoken.TokenDecoder(public_key),
        logger=None,
        daemon=False,
        web=None,
        target_host=None,
        target_port=None,
        wrap_mode='exit',
        wrap_cmd=None,
        RequestHandlerClass=TermideskProxyRequestHandler,
        **kwargs
    ).start_server()